from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource
from aiopoke.objects.utility.language import Language
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.pokemon import PokemonSpecies


class PokemonShape(NamedResource):
    awesome_names: List["AwesomeName"]
    names: List["Name"]
    pokemon_species: List[MinimalResource["PokemonSpecies"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        awesome_names: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
        pokemon_species: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.awesome_names = [
            AwesomeName(**awesome_name) for awesome_name in awesome_names
        ]
        self.names = [Name(**name) for name in names]
        self.pokemon_species = [
            MinimalResource(**pokemon_species) for pokemon_species in pokemon_species
        ]


class AwesomeName(Resource):
    awesome_name: str
    language: MinimalResource["Language"]

    def __init__(
        self,
        *,
        awesome_name: str,
        language: Dict[str, Any],
    ) -> None:
        self.awesome_name = awesome_name
        self.language = MinimalResource(**language)



import asyncio

import pytest

import aiopoke
from aiopoke.utils.resource import Resource


@pytest.mark.asyncio
async def test_endpoints(client: aiopoke.AiopokeClient):
    tasks = [
        client.get_ability(2),
        client.get_berry(2),
        client.get_berry_flavor(2),
        client.get_berry_firmness(2),
        client.get_characteristic(2),
        client.get_contest_effect(2),
        client.get_contest_type(2),
        client.get_egg_group(2),
        client.get_encounter_condition(2),
        client.get_encounter_condition_value(2),
        client.get_encounter_method(2),
        client.get_evolution_chain(2),
        client.get_evolution_trigger(2),
        client.get_gender(2),
        client.get_generation(2),
        client.get_growth_rate(2),
        client.get_item(2),
        client.get_item_attribute(2),
        client.get_item_category(2),
        client.get_item_fling_effect(2),
        client.get_item_pocket(2),
        client.get_language(2),
        client.get_location(2),
        client.get_location_area(2),
        client.get_machine(2),
        client.get_move(2),
        client.get_move_ailment(2),
        client.get_move_battle_style(2),
        client.get_move_category(2),
        client.get_move_damage_class(2),
        client.get_move_learn_method(2),
        client.get_move_target(2),
        client.get_nature(2),
        client.get_pal_park_area(2),
        client.get_pokeathlon_stat(2),
        client.get_pokedex(2),
        client.get_pokemon(2),
        client.get_pokemon_color(2),
        client.get_pokemon_form(2),
        client.get_pokemon_habitat(2),
        client.get_pokemon_shape(2),
        client.get_pokemon_species(2),
        client.get_region(2),
        client.get_stat(2),
        client.get_super_contest_effect(2),
        client.get_type(2),
        client.get_version(2),
        client.get_version_group(2),
    ]

    responses = await asyncio.gather(*tasks)
    for response in responses:
        assert isinstance(response, Resource)



from typing import Any
from typing import Dict
from typing import List
from typing import TYPE_CHECKING

from aiopoke.objects.utility.common_models import Description
from aiopoke.objects.utility.common_models import Name
from aiopoke.objects.utility.common_models import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources import VersionGroup


class MoveLearnMethod(NamedResource):
    descriptions: List["Description"]
    names: List["Name"]
    version_groups: List[MinimalResource["VersionGroup"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        descriptions: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
        version_groups: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.descriptions = [Description(**description) for description in descriptions]
        self.names = [Name(**name) for name in names]
        self.version_groups = [
            MinimalResource(**version_group) for version_group in version_groups
        ]



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models import Description, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources.moves import Move


class MoveCategory(NamedResource):
    descriptions: List["Description"]
    moves: List[MinimalResource["Move"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        descriptions: List[Dict[str, Any]],
        moves: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.descriptions = [Description(**description) for description in descriptions]
        self.moves = [MinimalResource(**move) for move in moves]



from aiopoke.objects.resources.berries.berry import Berry
from aiopoke.objects.resources.berries.berry_firmness import BerryFirmness
from aiopoke.objects.resources.berries.berry_flavor import BerryFlavor

__all__ = ("Berry", "BerryFirmness", "BerryFlavor")



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Effect, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources.items import Item


class ItemFlingEffect(NamedResource):
    effect_entries: List["Effect"]
    items: List[MinimalResource["Item"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        effect_entries: List[Dict[str, Any]],
        items: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.effect_entries = [
            Effect(**effect_entry) for effect_entry in effect_entries
        ]
        self.items = [MinimalResource(**item) for item in items]



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Name, NamedResource, Sprites
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import NaturalGiftType, VersionGroup
    from aiopoke.objects.resources.pokemon import Pokemon


class PokemonForm(NamedResource):
    form_name: str
    form_names: List["Name"]
    form_order: int
    is_battle_only: bool
    is_default: bool
    is_mega: bool
    order: int
    names: List["Name"]
    pokemon: MinimalResource["Pokemon"]
    sprites: "Sprites"
    types: List["SlotNaturalGiftType"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        form_name: str,
        form_names: List[Dict[str, Any]],
        form_order: int,
        is_battle_only: bool,
        is_default: bool,
        is_mega: bool,
        order: int,
        names: List[Dict[str, Any]],
        pokemon: Dict[str, Any],
        sprites: Dict[str, Any],
        types: List[Dict[str, Any]],
        version_group: Dict[str, Any],
    ) -> None:
        super().__init__(id=id, name=name)
        self.form_name = form_name
        self.form_names = [Name(**name) for name in form_names]
        self.form_order = form_order
        self.is_battle_only = is_battle_only
        self.is_default = is_default
        self.is_mega = is_mega
        self.order = order
        self.names = [Name(**name) for name in names]
        self.pokemon = MinimalResource(**pokemon)
        self.sprites = Sprites(sprites)
        self.types = [SlotNaturalGiftType(**type_) for type_ in types]
        self.version_group = MinimalResource(**version_group)


class SlotNaturalGiftType(Resource):
    type: MinimalResource[MinimalResource["NaturalGiftType"]]
    slot: int

    def __init__(
        self,
        *,
        type: Dict[str, Any],
        slot: int,
    ) -> None:
        self.type = MinimalResource(**type)
        self.slot = slot



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.utility import GenerationGameIndex
from aiopoke.objects.utility import MachineVersionDetail
from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.objects.utility import Sprite
from aiopoke.objects.utility import VerboseEffect
from aiopoke.objects.utility import VersionGroupFlavorText
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.minimal_resources import Url
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import EvolutionChain, Pokemon, Version
    from aiopoke.objects.resources.items import (
        ItemAttribute,
        ItemCategory,
        ItemFlingEffect,
    )


class Item(NamedResource):
    attributes: List[MinimalResource["ItemAttribute"]]
    baby_trigger_for: Optional[Url["EvolutionChain"]]
    category: MinimalResource["ItemCategory"]
    cost: int
    effect_entries: List["VerboseEffect"]
    flavor_text_entries: List["VersionGroupFlavorText"]
    fling_effect: Optional[MinimalResource["ItemFlingEffect"]]
    fling_power: Optional[int]
    game_indices: List["GenerationGameIndex"]
    held_by_pokemon: List["ItemHolderPokemon"]
    machines: List["MachineVersionDetail"]
    names: List["Name"]
    sprite: Optional["ItemSprites"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        attributes: List[Dict[str, Any]],
        baby_trigger_for: Optional[Dict[str, Any]],
        category: Dict[str, Any],
        cost: int,
        effect_entries: List[Dict[str, Any]],
        flavor_text_entries: List[Dict[str, Any]],
        fling_effect: Optional[Dict[str, Any]],
        fling_power: Optional[int],
        game_indices: List[Dict[str, Any]],
        held_by_pokemon: List[Dict[str, Any]],
        machines: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
        sprites: Optional[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.attributes = [MinimalResource(**attribute) for attribute in attributes]
        self.baby_trigger_for = (
            Url(**baby_trigger_for) if baby_trigger_for is not None else None
        )
        self.category = MinimalResource(**category)
        self.cost = cost
        self.effect_entries = [
            VerboseEffect(**effect_entry) for effect_entry in effect_entries
        ]
        self.flavor_text_entries = [
            VersionGroupFlavorText(**flavor_text_entry)
            for flavor_text_entry in flavor_text_entries
        ]
        self.fling_effect = Url(**fling_effect) if fling_effect is not None else None  # type: ignore
        self.fling_power = fling_power
        self.game_indices = [
            GenerationGameIndex(**game_index) for game_index in game_indices
        ]
        self.held_by_pokemon = [
            ItemHolderPokemon(**held_by_pokemon) for held_by_pokemon in held_by_pokemon
        ]
        self.machines = [MachineVersionDetail(**machine) for machine in machines]
        self.names = [Name(**name) for name in names]
        self.sprite = ItemSprites(**sprites) if sprites is not None else None


class ItemSprites(Resource):
    default: Optional["Sprite"]

    def __init__(self, *, default: Optional[str]) -> None:
        self.default = Sprite(default)


class ItemHolderPokemon(Resource):
    pokemon: MinimalResource["Pokemon"]
    version_details: List["ItemHolderPokemonVersionDetail"]

    def __init__(
        self,
        *,
        pokemon: Dict[str, Any],
        version_details: List[Dict[str, Any]],
    ) -> None:
        self.pokemon = MinimalResource(**pokemon)
        self.version_details = [
            ItemHolderPokemonVersionDetail(**version_detail)
            for version_detail in version_details
        ]


class ItemHolderPokemonVersionDetail(Resource):
    rarity: int
    version: MinimalResource["Version"]

    def __init__(self, *, rarity: int, version: Dict[str, Any]) -> None:
        self.rarity = rarity
        self.version = MinimalResource(**version)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Generation, Pokedex, VersionGroup
    from aiopoke.objects.resources.locations import Location


class Region(NamedResource):
    locations: List[MinimalResource["Location"]]
    main_generation: MinimalResource["Generation"]
    pokedexes: List[MinimalResource["Pokedex"]]
    names: List["Name"]
    version_groups: List[MinimalResource["VersionGroup"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        locations: List[Dict[str, Any]],
        main_generation: Dict[str, Any],
        pokedexes: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
        version_groups: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.locations = [MinimalResource(**location) for location in locations]
        self.main_generation = MinimalResource(**main_generation)
        self.pokedexes = [MinimalResource(**pokedex) for pokedex in pokedexes]
        self.names = [Name(**name) for name in names]
        self.version_groups = [
            MinimalResource(**version_group) for version_group in version_groups
        ]



from typing import Any
from typing import Dict
from typing import List
from typing import TYPE_CHECKING

from aiopoke.objects.resources.games.version_group import VersionGroupDetail
from aiopoke.objects.utility import NamedResource
from aiopoke.objects.utility import VersionEncounterDetail
from aiopoke.objects.utility import VersionGameIndex
from aiopoke.objects.utility.common_models.sprites import Sprites
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import (
        Generation,
        Item,
        LocationArea,
        MoveLearnMethod,
        Version,
        VersionGroup,
    )
    from aiopoke.objects.resources.pokemon import (
        Ability,
        NaturalGiftType,
        PokemonForm,
        PokemonSpecies,
    )


class Pokemon(NamedResource):
    abilities: List["PokemonAbility"]
    base_experience: int
    forms: List[MinimalResource["PokemonForm"]]
    game_indices: List["VersionGameIndex"]
    held_items: List["PokemonHeldItem"]
    height: int
    is_default: bool
    location_area_encounters: List["PokemonLocationArea"]
    moves: List["PokemonMove"]
    order: int
    past_abilities: List["PastAbility"]
    past_types: List["PastType"]
    species: MinimalResource["PokemonSpecies"]
    sprites: "Sprites"
    stats: List["PokemonStat"]
    types: List["PokemonType"]
    weight: int

    def __init__(
        self,
        *,
        id: int,
        name: str,
        abilities: List[Dict[str, Any]],
        base_experience: int,
        forms: List[Dict[str, Any]],
        game_indices: List[Dict[str, Any]],
        height: int,
        held_items: List[Dict[str, Any]],
        is_default: bool,
        location_area_encounters: List[Dict[str, Any]],
        moves: List[Dict[str, Any]],
        order: int,
        past_abilities: Dict[str, Any],
        past_types: List[Dict[str, Any]],
        species: Dict[str, Any],
        sprites: Dict[str, Any],
        stats: List[Dict[str, Any]],
        types: List[Dict[str, Any]],
        weight: int,
    ) -> None:
        super().__init__(id=id, name=name)
        self.abilities = [PokemonAbility(**ability) for ability in abilities]
        self.base_experience = base_experience
        self.forms = [MinimalResource(**form) for form in forms]
        self.game_indices = [
            VersionGameIndex(**game_index) for game_index in game_indices
        ]
        self.height = height
        self.held_items = [PokemonHeldItem(**held_item) for held_item in held_items]
        self.is_default = is_default
        self.location_area_encounters = [
            PokemonLocationArea(**location_area_encounter)
            for location_area_encounter in location_area_encounters
        ]
        self.moves = [PokemonMove(**move) for move in moves]
        self.order = order
        self.past_abilities = [PastAbility(**past_type) for past_type in past_types]
        self.past_types = [PastType(**past_type) for past_type in past_types]
        self.species = MinimalResource(**species)
        self.sprites = Sprites(sprites)
        self.stats = [PokemonStat(**stat) for stat in stats]
        self.types = [PokemonType(**type_) for type_ in types]
        self.weight = weight


class PokemonAbility(Resource):
    is_hidden: bool
    slot: int
    ability: MinimalResource["Ability"]

    def __init__(
        self,
        *,
        is_hidden: bool,
        slot: int,
        ability: Dict[str, Any],
    ) -> None:
        self.is_hidden = is_hidden
        self.slot = slot
        self.ability = MinimalResource(**ability)


class PokemonType(Resource):
    slot: int
    type: MinimalResource["NaturalGiftType"]

    def __init__(
        self,
        *,
        slot: int,
        type: Dict[str, Any],
    ) -> None:
        self.slot = slot
        self.type = MinimalResource(**type)


class PokemonHeldItem(Resource):
    item: MinimalResource["Item"]
    version_details: List["PokemonHeldItemVersion"]

    def __init__(
        self,
        *,
        item: Dict[str, Any],
        version_details: List[Dict[str, Any]],
    ) -> None:
        self.item = MinimalResource(**item)
        self.version_details = [
            PokemonHeldItemVersion(**version_detail)
            for version_detail in version_details
        ]


class PokemonHeldItemVersion(Resource):
    rarity: int
    version: MinimalResource["Version"]

    def __init__(
        self,
        *,
        rarity: int,
        version: Dict[str, Any],
    ) -> None:
        self.rarity = rarity
        self.version = MinimalResource(**version)


class PokemonMove(Resource):
    move: MinimalResource["Version"]
    version_group_details: List["VersionGroupDetail"]

    def __init__(
        self,
        *,
        move: Dict[str, Any],
        version_group_details: List[Dict[str, Any]],
    ) -> None:
        self.move = MinimalResource(**move)
        self.version_group_details = [
            VersionGroupDetail(**version_group_detail)
            for version_group_detail in version_group_details
        ]


class PokemonMoveVersion(Resource):
    move_learn_method: MinimalResource["MoveLearnMethod"]
    version_group: MinimalResource["VersionGroup"]
    level_learned_at: int

    def __init__(
        self,
        *,
        move_learn_method: Dict[str, Any],
        version_group: Dict[str, Any],
        level_learned_at: int,
    ) -> None:
        self.move_learn_method = MinimalResource(**move_learn_method)
        self.version_group = MinimalResource(**version_group)
        self.level_learned_at = level_learned_at


class PokemonStat(Resource):
    base_stat: int
    effort: int
    stat: MinimalResource["PokemonStat"]

    def __init__(
        self,
        *,
        base_stat: int,
        effort: int,
        stat: Dict[str, Any],
    ) -> None:
        self.base_stat = base_stat
        self.effort = effort
        self.stat = MinimalResource(**stat)


class PastAbility(Resource):
    abilities: List["PokemonAbility"]
    generation: MinimalResource["Generation"]

    def __init__(self, *, abilities: List[Dict[str, Any]], generation: Dict[str, Any]) -> None:
        self.ability = [PokemonAbility(**ability) for ability in abilities]
        self.generation = MinimalResource(**generation)


class PastType(Resource):
    generation: MinimalResource["Generation"]
    types: List["PokemonType"]

    def __init__(
        self,
        *,
        generation: Dict[str, Any],
        types: List[Dict[str, Any]],
    ) -> None:
        self.generation = MinimalResource(**generation)
        self.types = [PokemonType(**type_) for type_ in types]


class PokemonLocationArea(Resource):
    location_area: MinimalResource["LocationArea"]
    version_details: List["VersionEncounterDetail"]

    def __init__(
        self,
        *,
        location_area: Dict[str, Any],
        version_details: List[Dict[str, Any]],
    ) -> None:
        self.location_area = MinimalResource(**location_area)
        self.version_details = [
            VersionEncounterDetail(**version_detail)
            for version_detail in version_details
        ]



from typing import TYPE_CHECKING, Any, Dict

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import VersionGroup
    from aiopoke.objects.utility import Language


class VersionGroupFlavorText(Resource):
    text: str
    language: MinimalResource["Language"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self, *, text: str, language: Dict[str, Any], version_group: Dict[str, Any]
    ):
        self.text = text
        self.language = MinimalResource(**language)
        self.version_group = MinimalResource(**version_group)



import pytest

import aiopoke


@pytest.mark.asyncio
async def test_invalid_endpoint(client: aiopoke.AiopokeClient):
    with pytest.raises(ValueError):
        await client.get_egg_group(1234)

    with pytest.raises(ValueError):
        await client.get_egg_group("invalid")

    assert client.http.inexistent_endpoints == ["egg-group/1234", "egg-group/invalid"]



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import PokemonSpecies


class PalParkArea(NamedResource):
    pokemon_encounters: List["PalParkEncounterSpecies"]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        pokemon_encounters: List[Dict[str, Any]],
        names: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.pokemon_encounters = [
            PalParkEncounterSpecies(**pokemon_encounters)
            for pokemon_encounters in pokemon_encounters
        ]
        self.names = [Name(**name) for name in names]


class PalParkEncounterSpecies(Resource):
    base_score: int
    rate: int
    pokemon_species: MinimalResource["PokemonSpecies"]

    def __init__(
        self, base_score: int, rate: int, pokemon_species: Dict[str, Any]
    ) -> None:
        self.base_score = base_score
        self.rate = rate
        self.pokemon_species = MinimalResource(**pokemon_species)



from typing import Any
from typing import Dict
from typing import List
from typing import Optional

from aiohttp import ClientSession


class HttpClient:
    _session: ClientSession
    inexistent_endpoints: List[str]

    def __init__(self, *, session: Optional[ClientSession]) -> None:
        self._session = session or ClientSession()
        self.inexistent_endpoints = []

    async def close(self) -> None:
        if self._session is not None:
            await self._session.close()

    async def get(self, endpoint: str) -> Dict[str, Any]:
        if endpoint in self.inexistent_endpoints:
            raise ValueError(f"The id or name for {endpoint} was not found.")

        async with self._session.get(
            f"https://pokeapi.co/api/v2/{endpoint}"
        ) as response:
            if response.status == 404:
                self.inexistent_endpoints.append(endpoint)
                raise ValueError(f"The id or name for {endpoint} was not found.")

            data: Dict[str, Any] = await response.json()
            return data



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.utility.common_models import Name
from aiopoke.objects.utility.common_models import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import BerryFlavor, MoveBatteStyle
    from aiopoke.objects.resources.pokemon import PokeathlonStat
    from aiopoke.objects.resources.pokemon.stat import Stat


class Nature(NamedResource):
    decreased_stat: Optional[MinimalResource["Stat"]]
    increased_stat: Optional[MinimalResource["Stat"]]
    likes_flavor: Optional[MinimalResource["BerryFlavor"]]
    hates_flavor: Optional[MinimalResource["BerryFlavor"]]
    move_battle_style_preferences: List["MoveBattleStylePreference"]
    names: List["Name"]
    pokeathlon_stat_changes: List["NatureStatChange"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        decreased_stat: Optional[Dict[str, Any]],
        hates_flavor: Optional[Dict[str, Any]],
        increased_stat: Optional[Dict[str, Any]],
        likes_flavor: Optional[Dict[str, Any]],
        move_battle_style_preferences: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
        pokeathlon_stat_changes: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.decreased_stat = (
            MinimalResource(**decreased_stat) if decreased_stat is not None else None
        )
        self.hates_flavor = (
            MinimalResource(**hates_flavor) if hates_flavor is not None else None
        )
        self.increased_stat = (
            MinimalResource(**increased_stat) if increased_stat is not None else None
        )
        self.likes_flavor = (
            MinimalResource(**likes_flavor) if likes_flavor is not None else None
        )
        self.move_battle_style_preferences = [
            MoveBattleStylePreference(**move_battle_style_preference)
            for move_battle_style_preference in move_battle_style_preferences
        ]
        self.names = [Name(**name) for name in names]
        self.pokeathlon_stat_changes = [
            NatureStatChange(**pokeathlon_stat_change)
            for pokeathlon_stat_change in pokeathlon_stat_changes
        ]


class NatureStatChange(Resource):
    max_change: int
    pokeathlon_stat: MinimalResource["PokeathlonStat"]

    def __init__(
        self,
        *,
        max_change: int,
        pokeathlon_stat: Dict[str, Any],
    ) -> None:
        self.max_change = max_change
        self.pokeathlon_stat = MinimalResource(**pokeathlon_stat)


class MoveBattleStylePreference(Resource):
    low_hp_preference: int
    high_hp_preference: int
    move_battle_style: MinimalResource["MoveBatteStyle"]

    def __init__(
        self,
        *,
        low_hp_preference: int,
        high_hp_preference: int,
        move_battle_style: Dict[str, Any],
    ) -> None:
        self.low_hp_preference = low_hp_preference
        self.high_hp_preference = high_hp_preference
        self.move_battle_style = MinimalResource(**move_battle_style)



import os

import pytest

import aiopoke


@pytest.mark.asyncio
async def test_sprites(client: aiopoke.AiopokeClient):
    pokemon = await client.get_pokemon(25)
    sprites = pokemon.sprites

    await sprites.back_default.save()
    assert "back_25.png" in os.listdir()
    assert sprites.back_default.client == client



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources.encounters import EncounterCondition


class EncounterConditionValue(NamedResource):
    condition: MinimalResource["EncounterCondition"]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        condition: Dict[str, Any],
        names: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.condition = MinimalResource(**condition)
        self.names = [Name(**name) for name in names]



from typing import Any
from typing import Dict
from typing import List
from typing import TYPE_CHECKING

from aiopoke.objects.utility.common_models import Name
from aiopoke.objects.utility.common_models import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources import (
        Ability,
        Move,
        NaturalGiftType,
        PokemonSpecies,
        Region,
        VersionGroup,
    )


class Generation(NamedResource):
    abilities: List[MinimalResource["Ability"]]
    main_region: MinimalResource["Region"]
    moves: List[MinimalResource["Move"]]
    pokemon_species: List[MinimalResource["PokemonSpecies"]]
    types: List[MinimalResource["NaturalGiftType"]]
    version_groups: List[MinimalResource["VersionGroup"]]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        abilities: List[Dict[str, Any]],
        main_region: Dict[str, Any],
        moves: List[Dict[str, Any]],
        pokemon_species: List[Dict[str, Any]],
        types: List[Dict[str, Any]],
        version_groups: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.abilities = [MinimalResource(**ability) for ability in abilities]
        self.main_region = MinimalResource(**main_region)
        self.moves = [MinimalResource(**move) for move in moves]
        self.pokemon_species = [
            MinimalResource(**pokemon_species) for pokemon_species in pokemon_species
        ]
        self.types = [MinimalResource(**type_) for type_ in types]
        self.version_groups = [
            MinimalResource(**version_group) for version_group in version_groups
        ]
        self.names = [Name(**name) for name in names]



from typing import Any
from typing import Dict
from typing import List
from typing import TYPE_CHECKING

from aiopoke.objects.utility import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.berries import BerryFirmness, BerryFlavor
    from aiopoke.objects.resources.items.item import Item
    from aiopoke.objects.resources.pokemon.natural_gift_type import NaturalGiftType


class Berry(NamedResource):
    firmness: MinimalResource["BerryFirmness"]
    flavors: List["BerryFlavorMap"]
    item: MinimalResource["Item"]
    growth_time: int
    max_harvest: int
    natural_gift_power: int
    natural_gift_type: MinimalResource["NaturalGiftType"]
    soil_dryness: int
    smoothness: int
    size: int

    def __init__(
        self,
        *,
        id: int,
        name: str,
        firmness: Dict[str, Any],
        flavors: List[Dict[str, Any]],
        item: Dict[str, Any],
        growth_time: int,
        max_harvest: int,
        natural_gift_power: int,
        natural_gift_type: Dict[str, Any],
        soil_dryness: int,
        smoothness: int,
        size: int
    ) -> None:
        super().__init__(id=id, name=name)
        self.firmness = MinimalResource(**firmness)
        self.flavors = [BerryFlavorMap(**flavor) for flavor in flavors]
        self.item = MinimalResource(**item)
        self.growth_time = growth_time
        self.max_harvest = max_harvest
        self.natural_gift_power = natural_gift_power
        self.natural_gift_type = MinimalResource(**natural_gift_type)
        self.soil_dryness = soil_dryness
        self.smoothness = smoothness
        self.size = size


class BerryFlavorMap(Resource):
    potency: int
    flavor: MinimalResource["BerryFlavor"]

    def __init__(self, *, potency: int, flavor: Dict[str, Any]) -> None:
        self.potency = potency
        self.flavor = MinimalResource(**flavor)



from typing import TYPE_CHECKING, Any, Dict

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.utility import Language


class Name(Resource):
    name: str
    language: MinimalResource["Language"]

    def __init__(
        self,
        *,
        name: str,
        language: Dict[str, Any],
    ):
        self.name = name
        self.language = MinimalResource(**language)



from typing import Any
from typing import Dict
from typing import List
from typing import TYPE_CHECKING

from aiopoke.objects.utility.common_models import Description
from aiopoke.objects.utility.common_models import Name
from aiopoke.objects.utility.common_models import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources.moves import Move


class MoveTarget(NamedResource):
    descriptions: List["Description"]
    moves: List[MinimalResource["Move"]]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        descriptions: List[Dict[str, Any]],
        moves: List[Dict[str, Any]],
        names: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.descriptions = [Description(**description) for description in descriptions]
        self.moves = [MinimalResource(**move) for move in moves]
        self.names = [Name(**name) for name in names]



from typing import Optional

from aiopoke.objects.utility.common_models.sprite import Sprite


class Sprites:
    back_default: Optional["Sprite"]
    back_female: Optional["Sprite"]
    back_shiny: Optional["Sprite"]
    back_shiny_female: Optional["Sprite"]
    front_default: Optional["Sprite"]
    front_female: Optional["Sprite"]
    front_shiny: Optional["Sprite"]
    front_shiny_female: Optional["Sprite"]
    generation_1: Optional["GenerationISprites"]
    generation_2: Optional["GenerationIISprites"]
    generation_3: Optional["GenerationIIISprites"]
    generation_4: Optional["GenerationIVSprites"]
    generation_5: Optional["GenerationVSprites"]
    generation_6: Optional["GenerationVISprites"]
    generation_7: Optional["GenerationVIISprites"]
    generation_8: Optional["GenerationVIIISprites"]
    other: Optional["Other"]

    def __init__(self, data) -> None:
        self.back_default = Sprite(data["back_default"])
        self.back_female = Sprite(data["back_female"])
        self.back_shiny = Sprite(data["back_shiny"])
        self.back_shiny_female = Sprite(data["back_shiny_female"])

        self.front_default = Sprite(data["front_default"])
        self.front_female = Sprite(data["front_female"])
        self.front_shiny = Sprite(data["front_shiny"])
        self.front_shiny_female = Sprite(data["front_shiny_female"])
        if data.get("versions") is not None:
            self.generation_i = self.generation_1 = GenerationISprites(
                data["versions"]["generation-i"]
            )
            self.generation_ii = self.generation_2 = GenerationIISprites(
                data["versions"]["generation-ii"]
            )
            self.generation_iii = self.generation_3 = GenerationIIISprites(
                data["versions"]["generation-iii"]
            )
            self.generation_iv = self.generation_4 = GenerationIVSprites(
                data["versions"]["generation-iv"]
            )
            self.generation_v = self.generation_5 = GenerationVSprites(
                data["versions"]["generation-v"]
            )
            self.generation_vi = self.generation_6 = GenerationVISprites(
                data["versions"]["generation-vi"]
            )
            self.generation_vii = self.generation_7 = GenerationVIISprites(
                data["versions"]["generation-vii"]
            )
            self.generation_viii = self.generation_8 = GenerationVIIISprites(
                data["versions"]["generation-viii"]
            )
        self.other = Other(data["other"]) if data.get("other") is not None else None


class GenerationISprites:
    red_blue_back_default: Optional["Sprite"]
    red_blue_back_gray: Optional["Sprite"]
    red_blue_front_default: Optional["Sprite"]
    red_blue_front_gray: Optional["Sprite"]
    yellow_back_default: Optional["Sprite"]
    yellow_back_gray: Optional["Sprite"]
    yellow_front_default: Optional["Sprite"]
    yellow_front_gray: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.red_blue_back_default = Sprite(data["red-blue"]["back_default"])
        self.red_blue_back_gray = Sprite(data["red-blue"]["back_gray"])
        self.red_blue_front_default = Sprite(data["red-blue"]["front_default"])
        self.red_blue_front_gray = Sprite(data["red-blue"]["front_gray"])

        self.yellow_back_default = Sprite(data["yellow"]["back_default"])
        self.yellow_back_gray = Sprite(data["yellow"]["back_gray"])
        self.yellow_front_default = Sprite(data["yellow"]["front_default"])
        self.yellow_front_gray = Sprite(data["yellow"]["front_gray"])


class GenerationIISprites:
    crystal_back_default: Optional["Sprite"]
    crystal_back_shiny: Optional["Sprite"]
    crystal_front_default: Optional["Sprite"]
    crystal_front_shiny: Optional["Sprite"]
    gold_back_default: Optional["Sprite"]
    gold_back_shiny: Optional["Sprite"]
    gold_front_default: Optional["Sprite"]
    gold_front_shiny: Optional["Sprite"]
    silver_back_default: Optional["Sprite"]
    silver_back_shiny: Optional["Sprite"]
    silver_front_default: Optional["Sprite"]
    silver_front_shiny: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.crystal_back_default = Sprite(data["crystal"]["back_default"])
        self.crystal_back_shiny = Sprite(data["crystal"]["back_shiny"])
        self.crystal_front_default = Sprite(data["crystal"]["front_default"])
        self.crystal_front_shiny = Sprite(data["crystal"]["front_shiny"])

        self.gold_back_default = Sprite(data["gold"]["back_default"])
        self.gold_back_shiny = Sprite(data["gold"]["back_shiny"])
        self.gold_front_default = Sprite(data["gold"]["front_default"])
        self.gold_front_shiny = Sprite(data["gold"]["front_shiny"])

        self.silver_back_default = Sprite(data["silver"]["back_default"])
        self.silver_back_shiny = Sprite(data["silver"]["back_shiny"])
        self.silver_front_default = Sprite(data["silver"]["front_default"])
        self.silver_front_shiny = Sprite(data["silver"]["front_shiny"])


class GenerationIIISprites:
    emerald_front_default: Optional["Sprite"]
    emerald_front_shiny: Optional["Sprite"]
    firered_leafgreen_back_default: Optional["Sprite"]
    firered_leafgreen_back_shiny: Optional["Sprite"]
    firered_leafgreen_front_default: Optional["Sprite"]
    firered_leafgreen_front_gray: Optional["Sprite"]
    ruby_sapphire_back_default: Optional["Sprite"]
    ruby_sapphire_back_shiny: Optional["Sprite"]
    ruby_sapphire_front_default: Optional["Sprite"]
    ruby_sapphire_front_shiny: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.emerald_front_default = Sprite(data["emerald"]["front_default"])
        self.emerald_front_shiny = Sprite(data["emerald"]["front_shiny"])

        self.firered_leafgreen_back_default = Sprite(
            data["firered-leafgreen"]["back_default"]
        )
        self.firered_leafgreen_back_shiny = Sprite(
            data["firered-leafgreen"]["back_shiny"]
        )
        self.firered_leafgreen_front_default = Sprite(
            data["firered-leafgreen"]["front_default"]
        )
        self.firered_leafgreen_front_shiny = Sprite(
            data["firered-leafgreen"]["front_shiny"]
        )

        self.ruby_sapphire_back_default = Sprite(data["ruby-sapphire"]["back_default"])
        self.ruby_sapphire_back_shiny = Sprite(data["ruby-sapphire"]["back_shiny"])
        self.ruby_sapphire_front_default = Sprite(
            data["ruby-sapphire"]["front_default"]
        )
        self.ruby_sapphire_front_shiny = Sprite(data["ruby-sapphire"]["front_shiny"])


class GenerationIVSprites:
    diamond_pearl_back_default: Optional["Sprite"]
    diamond_pearl_back_female: Optional["Sprite"]
    diamond_pearl_back_shiny: Optional["Sprite"]
    diamond_pearl_back_shiny_female: Optional["Sprite"]
    diamond_pearl_front_default: Optional["Sprite"]
    diamond_pearl_front_female: Optional["Sprite"]
    diamond_pearl_front_shiny: Optional["Sprite"]
    diamond_pearl_front_shiny_female: Optional["Sprite"]
    heartgold_soulsilver_back_default: Optional["Sprite"]
    heartgold_soulsilver_back_female: Optional["Sprite"]
    heartgold_soulsilver_back_shiny: Optional["Sprite"]
    heartgold_soulsilver_back_shiny_female: Optional["Sprite"]
    heartgold_soulsilver_front_default: Optional["Sprite"]
    heartgold_soulsilver_front_female: Optional["Sprite"]
    heartgold_soulsilver_front_shiny: Optional["Sprite"]
    heartgold_soulsilver_front_shiny_female: Optional["Sprite"]
    platinum_back_default: Optional["Sprite"]
    platinum_back_female: Optional["Sprite"]
    platinum_back_shiny: Optional["Sprite"]
    platinum_back_shiny_female: Optional["Sprite"]
    platinum_front_default: Optional["Sprite"]
    platinum_front_female: Optional["Sprite"]
    platinum_front_shiny: Optional["Sprite"]
    platinum_front_shiny_female: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.diamond_pearl_back_default = Sprite(data["diamond-pearl"]["back_default"])
        self.diamond_pearl_back_female = Sprite(data["diamond-pearl"]["back_female"])
        self.diamond_pearl_back_shiny = Sprite(data["diamond-pearl"]["back_shiny"])
        self.diamond_pearl_back_shiny_female = Sprite(
            data["diamond-pearl"]["back_shiny_female"]
        )
        self.diamond_pearl_front_default = Sprite(
            data["diamond-pearl"]["front_default"]
        )
        self.diamond_pearl_front_female = Sprite(data["diamond-pearl"]["front_female"])
        self.diamond_pearl_front_shiny = Sprite(data["diamond-pearl"]["front_shiny"])
        self.diamond_pearl_front_shiny_female = Sprite(
            data["diamond-pearl"]["front_shiny_female"]
        )

        self.heartgold_soulsilver_back_default = Sprite(
            data["heartgold-soulsilver"]["back_default"]
        )
        self.heartgold_soulsilver_back_female = Sprite(
            data["heartgold-soulsilver"]["back_female"]
        )
        self.heartgold_soulsilver_back_shiny = Sprite(
            data["heartgold-soulsilver"]["back_shiny"]
        )
        self.heartgold_soulsilver_back_shiny_female = Sprite(
            data["heartgold-soulsilver"]["back_shiny_female"]
        )
        self.heartgold_soulsilver_front_default = Sprite(
            data["heartgold-soulsilver"]["front_default"]
        )
        self.heartgold_soulsilver_front_female = Sprite(
            data["heartgold-soulsilver"]["front_female"]
        )
        self.heartgold_soulsilver_front_shiny = Sprite(
            data["heartgold-soulsilver"]["front_shiny"]
        )
        self.heartgold_soulsilver_front_shiny_female = Sprite(
            data["heartgold-soulsilver"]["front_shiny_female"]
        )

        self.platinum_back_default = Sprite(data["platinum"]["back_default"])
        self.platinum_back_female = Sprite(data["platinum"]["back_female"])
        self.platinum_back_shiny = Sprite(data["platinum"]["back_shiny"])
        self.platinum_back_shiny_female = Sprite(data["platinum"]["back_shiny_female"])
        self.platinum_front_default = Sprite(data["platinum"]["front_default"])
        self.platinum_front_female = Sprite(data["platinum"]["front_female"])
        self.platinum_front_shiny = Sprite(data["platinum"]["front_shiny"])
        self.platinum_front_shiny_female = Sprite(
            data["platinum"]["front_shiny_female"]
        )


class GenerationVSprites:
    black_white_animated_back_default: Optional["Sprite"]
    black_white_animated_back_female: Optional["Sprite"]
    black_white_animated_back_shiny: Optional["Sprite"]
    black_white_animated_back_shiny_female: Optional["Sprite"]
    black_white_animated_front_default: Optional["Sprite"]
    black_white_animated_front_female: Optional["Sprite"]
    black_white_animated_front_shiny: Optional["Sprite"]
    black_white_animated_front_shiny_female: Optional["Sprite"]
    black_white_back_default: Optional["Sprite"]
    black_white_back_female: Optional["Sprite"]
    black_white_back_shiny: Optional["Sprite"]
    black_white_back_shiny_female: Optional["Sprite"]
    black_white_front_default: Optional["Sprite"]
    black_white_front_female: Optional["Sprite"]
    black_white_front_shiny: Optional["Sprite"]
    black_white_front_shiny_female: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.black_white_animated_back_default = Sprite(
            data["black-white"]["animated"]["back_default"]
        )
        self.black_white_animated_back_female = Sprite(
            data["black-white"]["animated"]["back_female"]
        )
        self.black_white_animated_back_shiny = Sprite(
            data["black-white"]["animated"]["back_shiny"]
        )
        self.black_white_animated_back_shiny_female = Sprite(
            data["black-white"]["animated"]["back_shiny_female"]
        )
        self.black_white_animated_front_default = Sprite(
            data["black-white"]["animated"]["front_default"]
        )
        self.black_white_animated_front_female = Sprite(
            data["black-white"]["animated"]["front_female"]
        )
        self.black_white_animated_front_shiny = Sprite(
            data["black-white"]["animated"]["front_shiny"]
        )
        self.black_white_animated_front_shiny_female = Sprite(
            data["black-white"]["animated"]["back_default"]
        )

        self.black_white_back_default = Sprite(data["black-white"]["back_default"])
        self.black_white_back_female = Sprite(data["black-white"]["back_female"])
        self.black_white_back_shiny = Sprite(data["black-white"]["back_shiny"])
        self.black_white_back_shiny_female = Sprite(
            data["black-white"]["back_shiny_female"]
        )
        self.black_white_front_default = Sprite(data["black-white"]["front_default"])
        self.black_white_front_female = Sprite(data["black-white"]["front_female"])
        self.black_white_front_shiny = Sprite(data["black-white"]["front_shiny"])
        self.black_white_front_shiny_female = Sprite(
            data["black-white"]["back_default"]
        )


class GenerationVISprites:
    omegaruby_alphasapphire_front_default: Optional["Sprite"]
    omegaruby_alphasapphire_front_female: Optional["Sprite"]
    omegaruby_alphasapphire_front_shiny: Optional["Sprite"]
    omegaruby_alphasapphire_front_shiny_female: Optional["Sprite"]
    x_y_front_default: Optional["Sprite"]
    x_y_front_female: Optional["Sprite"]
    x_y_front_shiny: Optional["Sprite"]
    x_y_front_shiny_female: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.omegaruby_alphasapphire_front_default = Sprite(
            data["omegaruby-alphasapphire"]["front_default"]
        )
        self.omegaruby_alphasapphire_front_female = Sprite(
            data["omegaruby-alphasapphire"]["front_female"]
        )
        self.omegaruby_alphasapphire_front_shiny = Sprite(
            data["omegaruby-alphasapphire"]["front_shiny"]
        )
        self.omegaruby_alphasapphire_front_shiny_female = Sprite(
            data["omegaruby-alphasapphire"]["front_shiny_female"]
        )

        self.x_y_front_default = Sprite(data["x-y"]["front_default"])
        self.x_y_front_female = Sprite(data["x-y"]["front_female"])
        self.x_y_front_shiny = Sprite(data["x-y"]["front_default"])
        self.x_y_front_shiny_female = Sprite(data["x-y"]["front_shiny_female"])


class GenerationVIISprites:
    icons_front_default: Optional["Sprite"]
    icons_front_female: Optional["Sprite"]
    ultra_sun_ultra_moon_front_default: Optional["Sprite"]
    ultra_sun_ultra_moon_front_female: Optional["Sprite"]
    ultra_sun_ultra_moon_front_shiny: Optional["Sprite"]
    ultra_sun_ultra_moon_front_shiny_female: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.icons_front_default = Sprite(data["icons"]["front_default"])
        self.icons_front_female = Sprite(data["icons"]["front_female"])

        self.ultra_sun_ultra_moon_front_default = Sprite(
            data["ultra-sun-ultra-moon"]["front_default"]
        )
        self.ultra_sun_ultra_moon_front_female = Sprite(
            data["ultra-sun-ultra-moon"]["front_female"]
        )
        self.ultra_sun_ultra_moon_front_shiny = Sprite(
            data["ultra-sun-ultra-moon"]["front_default"]
        )
        self.ultra_sun_ultra_moon_front_shiny_female = Sprite(
            data["ultra-sun-ultra-moon"]["front_shiny_female"]
        )


class GenerationVIIISprites:
    icons_front_default: Optional["Sprite"]
    icons_front_female: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.icons_front_default = Sprite(data["icons"]["front_default"])
        self.icons_front_female = Sprite(data["icons"]["front_female"])


class Other:
    dream_world_front_default: Optional["Sprite"]
    dream_world_front_female: Optional["Sprite"]
    home_front_default: Optional["Sprite"]
    home_front_female: Optional["Sprite"]
    home_front_shiny: Optional["Sprite"]
    home_front_shiny_female: Optional["Sprite"]

    def __init__(self, data) -> None:
        self.dream_world_front_default = Sprite(data["dream_world"]["front_default"])
        self.dream_world_front_female = Sprite(data["dream_world"]["front_female"])
        self.home_front_default = Sprite(data["home"]["front_default"])
        self.home_front_female = Sprite(data["home"]["front_female"])
        self.home_front_shiny = Sprite(data["home"]["front_shiny"])
        self.home_front_shiny_female = Sprite(data["home"]["front_shiny_female"])
        self.official_artwork_front_default = Sprite(
            data["official-artwork"]["front_default"]
        )
        self.official_artwork_front_shiny = Sprite(
            data["official-artwork"]["front_shiny"]
        )



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.resources.contests.super_contest_effect import SuperContestEffect
from aiopoke.objects.resources.pokemon.ability import AbilityEffectChange
from aiopoke.objects.utility import MachineVersionDetail
from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.objects.utility import VerboseEffect
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.minimal_resources import Url
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import (
        ContestEffect,
        ContestType,
        Generation,
        NaturalGiftType,
        Pokemon,
        Stat,
        VersionGroup,
    )
    from aiopoke.objects.resources.moves import (
        MoveAilment,
        MoveCategory,
        MoveDamageClass,
        MoveTarget,
    )
    from aiopoke.objects.utility import Language


class Move(NamedResource):
    accuracy: int
    contest_combos: Optional["ContestComboSets"]
    contest_effect: Optional[Url["ContestEffect"]]
    contest_type: Optional[MinimalResource["ContestType"]]
    damage_class: MinimalResource["MoveDamageClass"]
    effect_chance: Optional[int]
    effect_changes: List["AbilityEffectChange"]
    effect_entries: List["VerboseEffect"]
    flavor_text_entries: List["MoveFlavorText"]
    generation: MinimalResource["Generation"]
    learned_by_pokemon: List[MinimalResource["Pokemon"]]
    machines: List["MachineVersionDetail"]
    meta: Optional["MoveMetaData"]
    names: List["Name"]
    past_values: List["PastMoveStatValues"]
    power: int
    pp: int
    priority: int
    stat_changes: List["MoveStatChange"]
    super_contest_effect: Optional[Url["SuperContestEffect"]]
    target: MinimalResource["MoveTarget"]
    type: MinimalResource["NaturalGiftType"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        accuracy: int,
        contest_combos: Optional[Dict[str, Any]],
        contest_effect: Dict[str, Any],
        contest_type: Dict[str, Any],
        damage_class: Dict[str, Any],
        effect_chance: Optional[int],
        effect_entries: List[Dict[str, Any]],
        effect_changes: List[Dict[str, Any]],
        flavor_text_entries: List[Dict[str, Any]],
        generation: Dict[str, Any],
        learned_by_pokemon: List[Dict[str, Any]],
        machines: List[Dict[str, Any]],
        meta: Dict[str, Any],
        names: List[Dict[str, Any]],
        past_values: List[Dict[str, Any]],
        power: int,
        pp: int,
        priority: int,
        stat_changes: List[Dict[str, Any]],
        super_contest_effect: Dict[str, Any],
        target: Dict[str, Any],
        type: Dict[str, Any],
    ) -> None:
        super().__init__(id=id, name=name)
        self.accuracy = accuracy
        self.contest_combos = (
            ContestComboSets(**contest_combos) if contest_combos is not None else None
        )
        self.contest_effect = Url(**contest_effect)
        self.contest_type = MinimalResource(**contest_type)
        self.damage_class = MinimalResource(**damage_class)
        self.effect_chance = effect_chance
        self.effect_entries = [
            VerboseEffect(**effect_entry) for effect_entry in effect_entries
        ]
        self.effect_changes = [
            AbilityEffectChange(**effect_change) for effect_change in effect_changes
        ]
        self.flavor_text_entries = [
            MoveFlavorText(**flavor_text_entry)
            for flavor_text_entry in flavor_text_entries
        ]
        self.generation = MinimalResource(**generation)
        self.learned_by_pokemon = [
            MinimalResource(**pokemon) for pokemon in learned_by_pokemon
        ]
        self.machines = [MachineVersionDetail(**machine) for machine in machines]
        self.meta = MoveMetaData(**meta)
        self.names = [Name(**name) for name in names]
        self.past_values = [
            PastMoveStatValues(**past_value) for past_value in past_values
        ]
        self.power = power
        self.pp = pp
        self.priority = priority
        self.stat_changes = [
            MoveStatChange(**stat_change) for stat_change in stat_changes
        ]
        self.super_contest_effect = Url(**super_contest_effect)
        self.target = MinimalResource(**target)
        self.type = MinimalResource(**type)


class ContestComboSets(Resource):
    normal: Optional["ContestComboDetail"]
    super: Optional["ContestComboDetail"]

    def __init__(
        self,
        *,
        normal: Optional[Dict[str, Any]],
        super: Optional[Dict[str, Any]],
    ) -> None:
        self.normal = ContestComboDetail(**normal) if normal is not None else None
        self.super = ContestComboDetail(**super) if super is not None else None


class ContestComboDetail(Resource):
    use_before: Optional[List[MinimalResource["Move"]]]
    use_after: Optional[List[MinimalResource["Move"]]]

    def __init__(
        self,
        *,
        use_before: Optional[List[Dict[str, Any]]],
        use_after: Optional[List[Dict[str, Any]]],
    ) -> None:
        self.use_before = (
            [MinimalResource(**move) for move in use_before]
            if use_before is not None
            else None
        )
        self.use_after = (
            [MinimalResource(**move) for move in use_after]
            if use_after is not None
            else None
        )


class MoveFlavorText(Resource):
    flavor_text: str
    language: MinimalResource["Language"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self,
        *,
        flavor_text: str,
        language: Dict[str, Any],
        version_group: Dict[str, Any],
    ) -> None:
        self.flavor_text = flavor_text
        self.language = MinimalResource(**language)
        self.version_group = MinimalResource(**version_group)


class MoveMetaData(Resource):
    ailment: MinimalResource["MoveAilment"]
    category: MinimalResource["MoveCategory"]
    min_hits: int
    max_hits: int
    min_turns: int
    max_turns: int
    drain: int
    healing: int
    crit_rate: int
    ailment_chance: int
    flinch_chance: int
    stat_chance: int

    def __init__(
        self,
        *,
        ailment: Dict[str, Any],
        category: Dict[str, Any],
        min_hits: int,
        max_hits: int,
        min_turns: int,
        max_turns: int,
        drain: int,
        healing: int,
        crit_rate: int,
        ailment_chance: int,
        flinch_chance: int,
        stat_chance: int,
    ) -> None:
        self.ailment = MinimalResource(**ailment)
        self.category = MinimalResource(**category)
        self.min_hits = min_hits
        self.max_hits = max_hits
        self.min_turns = min_turns
        self.max_turns = max_turns
        self.drain = drain
        self.healing = healing
        self.crit_rate = crit_rate
        self.ailment_chance = ailment_chance
        self.flinch_chance = flinch_chance
        self.stat_chance = stat_chance


class MoveStatChange(Resource):
    change: int
    stat: MinimalResource["Stat"]

    def __init__(
        self,
        *,
        change: int,
        stat: Dict[str, Any],
    ) -> None:
        self.change = change
        self.stat = MinimalResource(**stat)


class PastMoveStatValues(Resource):
    accuracy: int
    effect_chance: int
    power: int
    pp: int
    effect_entries: List["VerboseEffect"]
    type: Optional[MinimalResource["NaturalGiftType"]]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self,
        *,
        accuracy: int,
        effect_chance: int,
        power: int,
        pp: int,
        effect_entries: List[Dict[str, Any]],
        type: Dict[str, Any],
        version_group: Dict[str, Any],
    ) -> None:
        self.accuracy = accuracy
        self.effect_chance = effect_chance
        self.power = power
        self.pp = pp
        self.effect_entries = [
            VerboseEffect(**effect_entry) for effect_entry in effect_entries
        ]
        self.type = MinimalResource(**type)
        self.version_group = MinimalResource(**version_group)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Description
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.pokemon import Stat


class Characteristic(Resource):
    descriptions: List["Description"]
    gene_modulo: int
    highest_stat: MinimalResource["Stat"]
    id: int
    possible_values: List[int]

    def __init__(
        self,
        *,
        descriptions: List[Dict[str, Any]],
        gene_modulo: int,
        highest_stat: Dict[str, Any],
        id: int,
        possible_values: List[int]
    ) -> None:
        self.descriptions = [Description(**description) for description in descriptions]
        self.gene_modulo = gene_modulo
        self.highest_stat = MinimalResource(**highest_stat)
        self.id = id
        self.possible_values = possible_values



from typing import TYPE_CHECKING, Any, Dict

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.utility import Language


class VerboseEffect(Resource):
    effect: str
    short_effect: str
    language: MinimalResource["Language"]

    def __init__(self, *, effect: str, short_effect: str, language: Dict[str, Any]):
        self.effect = effect
        self.short_effect = short_effect
        self.language = MinimalResource(**language)



from typing import Any
from typing import Callable
from typing import Coroutine
from typing import Dict
from typing import Optional
from typing import TYPE_CHECKING
from typing import TypeVar
from typing import Union

if TYPE_CHECKING:
    from aiopoke.aiopoke_client import AiopokeClient

U = TypeVar("U")


# cache decorator
def cache(
    endpoint: str,
    coro: Callable[["AiopokeClient", Union[str, int]], Coroutine[Any, Any, U]],
) -> Callable[["AiopokeClient", Union[str, int]], Coroutine[Any, Any, U]]:
    async def wrapper(client: "AiopokeClient", name_or_id: Union[str, int]) -> U:
        cached_item: Optional[U] = client._cache.get(f"{endpoint}/{name_or_id}")

        if cached_item:
            return cached_item

        obj: U = await coro(client, name_or_id)
        client._cache.put(f"{endpoint}/{name_or_id}", obj)
        return obj

    return wrapper


class Cache:
    cache: Dict[str, Any] = {}

    def get(self, id: Union[str, int]) -> Any:
        return self.cache.get(str(id))

    def put(self, id: Union[str, int], obj: Any) -> None:
        self.cache[str(id)] = obj

    def has(self, obj: Any) -> bool:
        return obj in self.cache.values()



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.utility import GenerationGameIndex
from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources.locations import LocationArea, Region


class Location(NamedResource):
    areas: List[MinimalResource["LocationArea"]]
    game_indices: List["GenerationGameIndex"]
    region: Optional[MinimalResource["Region"]]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        areas: List[Dict[str, Any]],
        game_indices: List[Dict[str, Any]],
        region: Dict[str, Any],
        names: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.areas = [MinimalResource(**area) for area in areas]
        self.game_indices = [
            GenerationGameIndex(**game_index) for game_index in game_indices
        ]
        self.region = MinimalResource(**region)
        self.names = [Name(**name) for name in names]



from aiopoke.objects.resources import Ability
from aiopoke.objects.resources import Berry
from aiopoke.objects.resources import BerryFirmness
from aiopoke.objects.resources import BerryFlavor
from aiopoke.objects.resources import Characteristic
from aiopoke.objects.resources import ContestEffect
from aiopoke.objects.resources import ContestType
from aiopoke.objects.resources import EggGroup
from aiopoke.objects.resources import EncounterCondition
from aiopoke.objects.resources import EncounterConditionValue
from aiopoke.objects.resources import EncounterMethod
from aiopoke.objects.resources import EvolutionChain
from aiopoke.objects.resources import EvolutionTrigger
from aiopoke.objects.resources import Gender
from aiopoke.objects.resources import Generation
from aiopoke.objects.resources import GrowthRate
from aiopoke.objects.resources import Item
from aiopoke.objects.resources import ItemAttribute
from aiopoke.objects.resources import ItemCategory
from aiopoke.objects.resources import ItemFlingEffect
from aiopoke.objects.resources import ItemPocket
from aiopoke.objects.resources import Location
from aiopoke.objects.resources import LocationArea
from aiopoke.objects.resources import Machine
from aiopoke.objects.resources import Move
from aiopoke.objects.resources import MoveAilment
from aiopoke.objects.resources import MoveBatteStyle
from aiopoke.objects.resources import MoveCategory
from aiopoke.objects.resources import MoveDamageClass
from aiopoke.objects.resources import MoveLearnMethod
from aiopoke.objects.resources import MoveTarget
from aiopoke.objects.resources import NaturalGiftType
from aiopoke.objects.resources import Nature
from aiopoke.objects.resources import PalParkArea
from aiopoke.objects.resources import PokeathlonStat
from aiopoke.objects.resources import Pokedex
from aiopoke.objects.resources import Pokemon
from aiopoke.objects.resources import PokemonColor
from aiopoke.objects.resources import PokemonForm
from aiopoke.objects.resources import PokemonHabitat
from aiopoke.objects.resources import PokemonShape
from aiopoke.objects.resources import PokemonSpecies
from aiopoke.objects.resources import Region
from aiopoke.objects.resources import Stat
from aiopoke.objects.resources import SuperContestEffect
from aiopoke.objects.resources import Version
from aiopoke.objects.resources import VersionGroup
from aiopoke.objects.utility import Description
from aiopoke.objects.utility import Effect
from aiopoke.objects.utility import Encounter
from aiopoke.objects.utility import FlavorText
from aiopoke.objects.utility import GenerationGameIndex
from aiopoke.objects.utility import Language
from aiopoke.objects.utility import MachineVersionDetail
from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.objects.utility import Sprite
from aiopoke.objects.utility import Sprites
from aiopoke.objects.utility import VerboseEffect
from aiopoke.objects.utility import VersionEncounterDetail
from aiopoke.objects.utility import VersionGameIndex
from aiopoke.objects.utility import VersionGroupFlavorText

__all__ = (
    "Berry",
    "BerryFirmness",
    "BerryFlavor",
    "ContestEffect",
    "ContestType",
    "SuperContestEffect",
    "EncounterCondition",
    "EncounterConditionValue",
    "EncounterMethod",
    "EvolutionChain",
    "EvolutionTrigger",
    "Generation",
    "Pokedex",
    "VersionGroup",
    "Version",
    "ItemAttribute",
    "ItemCategory",
    "ItemFlingEffect",
    "ItemPocket",
    "Item",
    "LocationArea",
    "Location",
    "PalParkArea",
    "Region",
    "Machine",
    "MoveAilment",
    "MoveBatteStyle",
    "MoveCategory",
    "MoveDamageClass",
    "MoveLearnMethod",
    "MoveTarget",
    "Move",
    "Ability",
    "Characteristic",
    "EggGroup",
    "Gender",
    "GrowthRate",
    "NaturalGiftType",
    "Nature",
    "PokeathlonStat",
    "PokemonColor",
    "PokemonForm",
    "PokemonHabitat",
    "PokemonShape",
    "PokemonSpecies",
    "Pokemon",
    "Stat",
    "Language",
    "Description",
    "Effect",
    "Encounter",
    "FlavorText",
    "GenerationGameIndex",
    "MachineVersionDetail",
    "Name",
    "NamedResource",
    "Sprite",
    "Sprites",
    "VerboseEffect",
    "VersionEncounterDetail",
    "VersionGameIndex",
    "VersionGroupFlavorText",
)



from aiopoke.utils.cache import Cache, cache
from aiopoke.utils.minimal_resources import MinimalResource, Url
from aiopoke.utils.resource import Resource

__all__ = (
    "Cache",
    "cache",
    "MinimalResource",
    "Url",
    "Resource",
)



from typing import Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource


class MoveBatteStyle(NamedResource):
    names: List["Name"]

    def __init__(self, *, id: int, name: str, names: List[Dict[str, Any]]) -> None:
        super().__init__(id=id, name=name)
        self.names = [Name(**name) for name in names]



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.utility import Description
from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import PokemonSpecies, Region, VersionGroup


class Pokedex(NamedResource):
    descriptions: List["Description"]
    is_main_series: bool
    pokemon_entries: List["PokemonEntry"]
    region: Optional[MinimalResource["Region"]]
    version_groups: List[MinimalResource["VersionGroup"]]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        descriptions: List[Dict[str, Any]],
        is_main_series: bool,
        pokemon_entries: List[Dict[str, Any]],
        region: Optional[Dict[str, Any]],
        version_groups: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.descriptions = [Description(**description) for description in descriptions]
        self.is_main_series = is_main_series
        self.pokemon_entries = [
            PokemonEntry(**pokemon_entry) for pokemon_entry in pokemon_entries
        ]
        self.region = MinimalResource(**region) if region is not None else None
        self.version_groups = [
            MinimalResource(**version_group) for version_group in version_groups
        ]
        self.names = [Name(**name) for name in names]


class PokemonEntry(Resource):
    entry_number: int
    pokemon_species: MinimalResource["PokemonSpecies"]

    def __init__(self, *, entry_number: int, pokemon_species: Dict[str, Any]) -> None:
        self.entry_number = entry_number
        self.pokemon_species = MinimalResource(**pokemon_species)



from aiopoke.objects.resources.evolutions.evolution_chain import EvolutionChain
from aiopoke.objects.resources.evolutions.evolution_trigger import EvolutionTrigger

__all__ = ("EvolutionChain", "EvolutionTrigger")



__title__ = "aiopoke"
__author__ = "beastmatser"
__license__ = "MIT"
__copyright__ = "Copyright 2021-present beastmatser"
__version__ = "0.1.7"

from typing import NamedTuple

from aiopoke.aiopoke_client import AiopokeClient
from aiopoke.objects import (
    Ability,
    Berry,
    BerryFirmness,
    BerryFlavor,
    Characteristic,
    ContestEffect,
    ContestType,
    Description,
    Effect,
    EggGroup,
    Encounter,
    EncounterCondition,
    EncounterConditionValue,
    EncounterMethod,
    EvolutionChain,
    EvolutionTrigger,
    FlavorText,
    Gender,
    Generation,
    GenerationGameIndex,
    GrowthRate,
    Item,
    ItemAttribute,
    ItemCategory,
    ItemFlingEffect,
    ItemPocket,
    Language,
    Location,
    LocationArea,
    Machine,
    MachineVersionDetail,
    Move,
    MoveAilment,
    MoveBatteStyle,
    MoveCategory,
    MoveDamageClass,
    MoveLearnMethod,
    MoveTarget,
    Name,
    NamedResource,
    NaturalGiftType,
    Nature,
    PalParkArea,
    PokeathlonStat,
    Pokedex,
    Pokemon,
    PokemonColor,
    PokemonForm,
    PokemonHabitat,
    PokemonShape,
    PokemonSpecies,
    Region,
    Sprite,
    Sprites,
    Stat,
    SuperContestEffect,
    VerboseEffect,
    Version,
    VersionEncounterDetail,
    VersionGameIndex,
    VersionGroup,
    VersionGroupFlavorText,
)


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str  # must be one of the following: alpha, beta, candidate, or final

    def __repr__(self) -> str:
        return f"{self.major}.{self.minor}.{self.micro}-{self.releaselevel}"


version_info: VersionInfo = VersionInfo(0, 1, 7, "final")


__all__ = (
    "Language",
    "Description",
    "Effect",
    "Encounter",
    "FlavorText",
    "GenerationGameIndex",
    "MachineVersionDetail",
    "Name",
    "NamedResource",
    "Sprite",
    "Sprites",
    "VerboseEffect",
    "VersionEncounterDetail",
    "VersionGameIndex",
    "VersionGroupFlavorText",
    "Berry",
    "BerryFirmness",
    "BerryFlavor",
    "ContestEffect",
    "ContestType",
    "SuperContestEffect",
    "EncounterCondition",
    "EncounterConditionValue",
    "EncounterMethod",
    "EvolutionChain",
    "EvolutionTrigger",
    "Generation",
    "Pokedex",
    "VersionGroup",
    "Version",
    "ItemAttribute",
    "ItemCategory",
    "ItemFlingEffect",
    "ItemPocket",
    "Item",
    "LocationArea",
    "Location",
    "PalParkArea",
    "Region",
    "Machine",
    "MoveAilment",
    "MoveBatteStyle",
    "MoveCategory",
    "MoveDamageClass",
    "MoveLearnMethod",
    "MoveTarget",
    "Move",
    "Ability",
    "Characteristic",
    "EggGroup",
    "Gender",
    "GrowthRate",
    "NaturalGiftType",
    "Nature",
    "PokeathlonStat",
    "PokemonColor",
    "PokemonForm",
    "PokemonHabitat",
    "PokemonShape",
    "PokemonSpecies",
    "Pokemon",
    "Stat",
    "AiopokeClient",
    "version_info",
    "__title__",
    "__author__",
    "__license__",
    "__copyright__",
    "__version__",
)



from typing import Any
from typing import Dict
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Version
    from aiopoke.objects.utility import Language


class FlavorText(Resource):
    flavor_text: str
    language: MinimalResource["Language"]
    version: Optional[MinimalResource["Version"]]

    def __init__(
        self,
        *,
        flavor_text: str,
        language: Dict[str, Any],
        version: Optional[Dict[str, Any]] = None,
    ):
        self.flavor_text = flavor_text
        self.language = MinimalResource(**language)
        self.version = MinimalResource(**version) if version else None



from typing import Any
from typing import Dict
from typing import List
from typing import TYPE_CHECKING

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import EncounterConditionValue, EncounterMethod


class Encounter(Resource):
    min_level: int
    max_level: int
    condition_values: List[MinimalResource["EncounterConditionValue"]]
    chance: int
    method: MinimalResource["EncounterMethod"]

    def __init__(
        self,
        *,
        min_level: int,
        max_level: int,
        condition_values: List[Dict[str, Any]],
        chance: int,
        method: Dict[str, Any],
    ):
        self.min_level = min_level
        self.max_level = max_level
        self.condition_values = [
            MinimalResource(**condition_value) for condition_value in condition_values
        ]
        self.chance = chance
        self.method = MinimalResource(**method)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models.encounter import Encounter
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Version


class VersionEncounterDetail(Resource):
    encounter_details: List["Encounter"]
    max_chance: int
    version: MinimalResource["Version"]

    def __init__(
        self,
        *,
        encounter_details: List[Dict[str, Any]],
        max_chance: int,
        version: Dict[str, Any]
    ):
        self.encounter_details = [
            Encounter(**encounter_detail) for encounter_detail in encounter_details
        ]
        self.max_chance = max_chance
        self.version = MinimalResource(**version)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources.berries import BerryFlavor


class ContestType(NamedResource):
    berry_flavor: MinimalResource["BerryFlavor"]
    names: List["ContestName"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        berry_flavor: Dict[str, Any],
        names: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.berry_flavor = MinimalResource(**berry_flavor)
        self.names = [ContestName(**name) for name in names]


class ContestName(Name):
    color: str

    def __init__(self, *, color: str, name: str, language: Dict[str, Any]) -> None:
        super().__init__(name=name, language=language)
        self.color = color



from os import getcwd
from typing import Optional
from typing import TYPE_CHECKING

import aiofiles

if TYPE_CHECKING:
    from aiopoke.aiopoke_client import AiopokeClient


def is_null(func):
    def wrapper(self, url):
        if url is None:
            return None

        return func(self, url)

    return wrapper


class Sprite:
    url: str
    bytes_: Optional[bytes]

    _client: Optional["AiopokeClient"] = None

    @is_null
    def __init__(self, url: str) -> None:
        self.url = url
        self.file_extention = url[url.rfind(".") + 1 :]  # noqa: E203
        self.bytes_ = None

    def __repr__(self) -> str:
        return f"Sprite(url={self.url})"

    @property
    def client(self) -> Optional["AiopokeClient"]:
        if hasattr(self, "_client"):
            return self._client

        return None

    @classmethod
    def link(cls, client: "AiopokeClient"):
        cls._client = client

    async def save(
        self, *, client: Optional["AiopokeClient"] = None, path: Optional[str] = None
    ) -> None:
        client = self.client or client
        if client is None:
            raise ValueError(
                "A client must be provided, if you create your own instances of this class"
            )

        if path is None:
            path = getcwd()

        elif not isinstance(path, str):
            raise ValueError("path must be a string")

        if "." in path.split("/")[-1]:
            raise ValueError(
                "You can not set the file extension, this is to avoid file corruption. Use os.system() instead"
            )

        if self.bytes_ is None:
            bytes_ = await self.read(client=client)

        file_name = path + "/" + "_".join(self.url.split("/")[-2:])[:-4] + ".png"
        async with aiofiles.open(file_name, "wb") as image:
            await image.write(bytes_)

    async def read(self, *, client: Optional["AiopokeClient"] = None) -> bytes:
        client = self.client or client
        if client is None:
            raise ValueError(
                "A client must be provided, if you create your own instances of this class"
            )

        async with client.http._session.get(self.url) as response:
            bytes_: bytes = await response.read()
            self.bytes_ = bytes_

        return bytes_
